@@ -19,6 +19,7 @@
 
 #define G_LOG_DOMAIN "GcalEditDialog"
 
+#include "gcal-alarm-row.h"
 #include "gcal-context.h"
 #include "gcal-date-selector.h"
 #include "gcal-debug.h"
@@ -28,6 +29,7 @@
 #include "gcal-event.h"
 #include "gcal-recurrence.h"
 
+#include <dazzle.h>
 #include <libecal/libecal.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -45,7 +47,7 @@
 
 struct _GcalEditDialog
 {
-  GtkDialog         parent;
+  HdyDialog         parent;
 
   gboolean          writable;
 
@@ -71,22 +73,24 @@ struct _GcalEditDialog
   GtkWidget        *summary_entry;
 
   GtkWidget        *start_date_selector;
+  GtkLabel         *event_start_label;
   GtkWidget        *end_date_selector;
-  GtkWidget        *all_day_check;
+  GtkLabel         *event_end_label;
+  GtkSwitch        *all_day_switch;
   GtkWidget        *start_time_selector;
   GtkWidget        *end_time_selector;
   GtkWidget        *location_entry;
   GtkWidget        *notes_text;
 
   GtkWidget        *alarms_listbox;
+  GtkPopover       *alarms_popover;
+  GtkListBoxRow    *new_alarm_row;
 
   GtkWidget        *repeat_combo;
   GtkWidget        *repeat_duration_combo;
-  GtkWidget        *repeat_limits_box;
 
   /* Recurrence widgets */
   GtkWidget        *number_of_occurrences_spin;
-  GtkWidget        *repeat_duration_stack;
   GtkWidget        *until_date_selector;
 
   /* Add Alarms popover buttons */
@@ -106,6 +110,7 @@ struct _GcalEditDialog
   /* new data holders */
   GcalEvent        *event;
   GcalCalendar     *selected_calendar;
+  GPtrArray        *alarms;
 
   /* flags */
   gboolean          event_is_new;
@@ -117,9 +122,6 @@ static void          on_calendar_selected_action_cb              (GSimpleAction
                                                                   GVariant           *value,
                                                                   gpointer            user_data);
 
-static void          on_sound_toggle_changed_cb                  (GtkToggleButton    *button,
-                                                                  GtkWidget          *row);
-
 static void          on_summary_entry_changed_cb                 (GtkEntry           *entry,
                                                                   GParamSpec         *pspec,
                                                                   GcalEditDialog     *self);
@@ -128,13 +130,10 @@ static void          on_location_entry_changed_cb                (GtkEntry
                                                                   GParamSpec         *pspec,
                                                                   GcalEditDialog     *self);
 
-static void          on_remove_alarm_button_clicked              (GtkButton          *button,
-                                                                  GtkWidget          *row);
-
 static void          on_add_alarm_button_clicked_cb              (GtkWidget          *button,
                                                                   GcalEditDialog     *self);
 
-G_DEFINE_TYPE (GcalEditDialog, gcal_edit_dialog, GTK_TYPE_DIALOG)
+G_DEFINE_TYPE (GcalEditDialog, gcal_edit_dialog, HDY_TYPE_DIALOG)
 
 enum
 {
@@ -238,7 +237,7 @@ find_best_timezones_for_event (GcalEditDialog  *self,
   gboolean all_day;
 
   /* Update all day */
-  all_day = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->all_day_check));
+  all_day = gtk_switch_get_active (self->all_day_switch);
   was_all_day = gcal_event_get_all_day (self->event);
 
   GCAL_TRACE_MSG ("Finding best timezone with all_day=%d, was_all_day=%d, event_is_new=%d",
@@ -305,7 +304,7 @@ return_datetime_for_widgets (GcalEditDialog   *self,
   GDateTime *retval;
   gboolean all_day;
 
-  all_day = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->all_day_check));
+  all_day = gtk_switch_get_active (self->all_day_switch);
   date = gcal_date_selector_get_date (date_selector);
   time = gcal_time_selector_get_time (time_selector);
 
@@ -367,7 +366,7 @@ set_writable (GcalEditDialog *self,
   if (self->writable == writable)
     return;
 
-  all_day = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->all_day_check));
+  all_day = gtk_switch_get_active (self->all_day_switch);
 
   gtk_widget_set_sensitive (self->start_time_selector, !all_day && writable);
   gtk_widget_set_sensitive (self->end_time_selector, !all_day && writable);
@@ -390,6 +389,114 @@ remove_recurrence_properties (GcalEvent *event)
   e_cal_component_set_rrules (comp, NULL);
 }
 
+static gchar*
+format_datetime_for_display (GDateTime      *date,
+                             GcalTimeFormat  format,
+                             gboolean        all_day)
+{
+  g_autofree gchar *formatted_date = NULL;
+  g_autoptr (GDateTime) now = NULL;
+  GString *string;
+  gint days_diff;
+
+  string = g_string_new ("");
+
+  now = g_date_time_new_now_local ();
+  days_diff = gcal_date_time_compare_date (now, date);
+
+  switch (days_diff)
+    {
+    case -7:
+    case -6:
+    case -5:
+    case -4:
+    case -3:
+    case -2:
+      /* Translators: %A is the weekday name (e.g. Sunday, Monday, etc) */
+      formatted_date = g_date_time_format (date, _("Last %A"));
+      break;
+
+    case -1:
+      formatted_date = g_strdup (_("Yesterday"));
+      break;
+
+    case 0:
+      formatted_date = g_strdup (_("Today"));
+      break;
+
+    case 1:
+      formatted_date = g_strdup (_("Tomorrow"));
+      break;
+
+    case 2:
+    case 3:
+    case 4:
+    case 5:
+    case 6:
+    case 7:
+      /* Translators: %A is the weekday name (e.g. Sunday, Monday, etc) */
+      formatted_date = g_date_time_format (date, _("This %A"));
+      break;
+
+    default:
+      formatted_date = g_date_time_format (date, "%x");
+      break;
+    }
+
+  if (!all_day)
+    {
+      g_autofree gchar *formatted_time = NULL;
+
+      switch (format)
+        {
+        case GCAL_TIME_FORMAT_12H:
+          formatted_time = g_date_time_format (date, "%I:%M %P");
+          break;
+
+        case GCAL_TIME_FORMAT_24H:
+          formatted_time = g_date_time_format (date, "%R");
+          break;
+
+        default:
+          g_assert_not_reached ();
+        }
+
+      /*
+       * Translators: %1$s is the formatted date (e.g. Today, Sunday, or even 2019-10-11) and %2$s is the
+       * formatted time (e.g. 03:14 PM, or 21:29)
+       */
+      g_string_printf (string, _("%1$s, %2$s"), formatted_date, formatted_time);
+    }
+  else
+    {
+      g_string_printf (string, "%s", formatted_date);
+    }
+
+  return g_string_free (string, FALSE);
+}
+
+static void
+update_date_labels (GcalEditDialog *self)
+{
+  g_autofree gchar *start_label = NULL;
+  g_autofree gchar *end_label = NULL;
+  g_autoptr (GDateTime) start = NULL;
+  g_autoptr (GDateTime) end = NULL;
+  GcalTimeFormat time_format;
+  gboolean all_day;
+
+  time_format = gcal_context_get_time_format (self->context);
+
+  all_day = gtk_switch_get_active (self->all_day_switch);
+  start = get_date_start (self);
+  end = get_date_end (self);
+  start_label = format_datetime_for_display (start, time_format, all_day);
+  end_label = format_datetime_for_display (end, time_format, all_day);
+
+  gtk_label_set_label (self->event_start_label, start_label);
+  gtk_label_set_label (self->event_end_label, end_label);
+}
+
 static void
 sync_datetimes (GcalEditDialog *self,
                 GParamSpec     *pspec,
@@ -402,7 +509,7 @@ sync_datetimes (GcalEditDialog *self,
   GCAL_ENTRY;
 
   is_start = (widget == self->start_time_selector || widget == self->start_date_selector);
-  is_all_day = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->all_day_check));
+  is_all_day = gtk_switch_get_active (self->all_day_switch);
   start = get_date_start (self);
   end = get_date_end (self);
 
@@ -449,6 +556,8 @@ out:
   g_clear_pointer (&start, g_date_time_unref);
   g_clear_pointer (&end, g_date_time_unref);
 
+  update_date_labels (self);
+
   GCAL_EXIT;
 }
 
@@ -506,110 +615,44 @@ get_row_for_alarm_trigger_minutes (GcalEditDialog *self,
   return NULL;
 }
 
-static GtkWidget*
-create_row_for_alarm (GcalEvent          *event,
-                      ECalComponentAlarm *alarm)
+static ECalComponentAlarm*
+create_alarm (guint minutes)
 {
-  ECalComponentAlarmAction action;
-  GtkBuilder *builder;
-  GtkWidget *label, *main_box, *row, *remove_button;
-  GtkWidget *volume_button, *volume_icon;
-  gboolean has_sound;
-  gchar *text;
-  gint trigger_minutes;
 
-  trigger_minutes = get_alarm_trigger_minutes (event, alarm);
+  ECalComponentAlarmTrigger *trigger;
+  ECalComponentAlarm *alarm;
+  ICalDuration *duration;
 
-  /* Something bad happened */
-  if (trigger_minutes < 0)
-    return NULL;
+  duration = i_cal_duration_new_null_duration ();
+  i_cal_duration_set_is_neg (duration, TRUE);
+  i_cal_duration_set_minutes (duration, minutes);
 
-  if (trigger_minutes < 60)
-    {
-      text = g_strdup_printf (g_dngettext (GETTEXT_PACKAGE,
-                                           "%d minute before",
-                                           "%d minutes before",
-                                           trigger_minutes),
-                              trigger_minutes);
-    }
-  else if (trigger_minutes < 1440)
-    {
-      text = g_strdup_printf (g_dngettext (GETTEXT_PACKAGE,
-                                           "%d hour before",
-                                           "%d hours before",
-                                           trigger_minutes / 60),
-                              trigger_minutes / 60);
-    }
-  else if (trigger_minutes < 10080)
-    {
-      text = g_strdup_printf (g_dngettext (GETTEXT_PACKAGE,
-                                           "%d day before",
-                                           "%d days before",
-                                           trigger_minutes / 1440),
-                              trigger_minutes / 1440);
-    }
-  else
-    {
-      text = g_strdup_printf (g_dngettext (GETTEXT_PACKAGE,
-                                           "%d week before",
-                                           "%d weeks before",
-                                           trigger_minutes / 10080),
-                              trigger_minutes / 10080);
-    }
+  trigger = e_cal_component_alarm_trigger_new_relative (E_CAL_COMPONENT_ALARM_TRIGGER_RELATIVE_START, duration);
 
-  /* The row */
-  row = gtk_list_box_row_new ();
-  gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (row), FALSE);
+  g_clear_object (&duration);
 
-  g_object_set_data (G_OBJECT (row), "alarm", alarm);
-  g_object_set_data (G_OBJECT (row), "event", event);
+  alarm = e_cal_component_alarm_new ();
+  e_cal_component_alarm_take_trigger (alarm, trigger);
+  e_cal_component_alarm_set_action (alarm, E_CAL_COMPONENT_ALARM_DISPLAY);
 
-  /* Build the UI */
-  builder = gtk_builder_new_from_resource ("/org/gnome/calendar/alarm-row.ui");
-
-#define WID(x) (GTK_WIDGET (gtk_builder_get_object (builder, x)))
-
-  label = WID ("label");
-  gtk_label_set_label (GTK_LABEL (label), text);
-
-  /* Retrieves the actions associated to the alarm */
-  action = e_cal_component_alarm_get_action (alarm);
-
-  /* Updates the volume button to match the action */
-  has_sound = action == E_CAL_COMPONENT_ALARM_AUDIO;
-
-  volume_button = WID ("volume_button");
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (volume_button), has_sound);
-
-  volume_icon = WID ("volume_icon");
-  gtk_image_set_from_icon_name (GTK_IMAGE (volume_icon),
-                                has_sound ? "audio-volume-high-symbolic" : "audio-volume-muted-symbolic",
-                                GTK_ICON_SIZE_BUTTON);
-
-  g_signal_connect_object (volume_button, "toggled", G_CALLBACK (on_sound_toggle_changed_cb), row, 0);
-
-  /* Remove button */
-  remove_button = WID ("remove_button");
-
-  g_signal_connect_object (remove_button,
-                           "clicked",
-                           G_CALLBACK (on_remove_alarm_button_clicked),
-                           row,
-                           0);
-
-  main_box = WID ("main_box");
-  gtk_container_add (GTK_CONTAINER (row), main_box);
-
-  gtk_widget_show_all (row);
-
-  g_clear_object (&builder);
-  g_free (text);
-
-#undef WID
-
-  return row;
+  return alarm;
 }
 
+static void
+clear_alarms (GcalEditDialog *self)
+{
+  g_autoptr (GList) children = NULL;
+  GList *l;
+
+  g_ptr_array_set_size (self->alarms, 0);
+
+  children = gtk_container_get_children (GTK_CONTAINER (self->alarms_listbox));
+  for (l = children; l != NULL; l = l->next)
+    {
+      if (l->data != self->new_alarm_row)
+        gtk_widget_destroy (l->data);
+    }
+}
 
 /*
  * Callbacks
@@ -698,17 +741,24 @@ sort_alarms_func (GtkListBoxRow *a,
                   GtkListBoxRow *b,
                   gpointer       user_data)
 {
-  ECalComponentAlarm *alarm_a, *alarm_b;
-  GcalEvent *event_a, *event_b;
-  gint minutes_a, minutes_b;
+  ECalComponentAlarm *alarm_a;
+  ECalComponentAlarm *alarm_b;
+  GcalEditDialog *self;
+  gint minutes_a;
+  gint minutes_b;
 
-  alarm_a = g_object_get_data (G_OBJECT (a), "alarm");
-  alarm_b = g_object_get_data (G_OBJECT (b), "alarm");
-  event_a = g_object_get_data (G_OBJECT (a), "event");
-  event_b = g_object_get_data (G_OBJECT (b), "event");
+  self = GCAL_EDIT_DIALOG (user_data);
 
-  minutes_a = get_alarm_trigger_minutes (event_a, alarm_a);
-  minutes_b = get_alarm_trigger_minutes (event_b, alarm_b);
+  if (a == self->new_alarm_row)
+    return 1;
+  else if (b == self->new_alarm_row)
+    return -1;
+
+  alarm_a = gcal_alarm_row_get_alarm (GCAL_ALARM_ROW (a));
+  minutes_a = get_alarm_trigger_minutes (self->event, alarm_a);
+
+  alarm_b = gcal_alarm_row_get_alarm (GCAL_ALARM_ROW (b));
+  minutes_b = get_alarm_trigger_minutes (self->event, alarm_b);
 
   return minutes_a - minutes_b;
 }
@@ -748,6 +798,7 @@ on_action_button_clicked_cb (GtkWidget *widget,
       gboolean was_all_day;
       gboolean all_day;
       gchar *note_text;
+      gsize i;
 
       /* Update summary */
       gcal_event_set_summary (self->event, gtk_entry_get_text (GTK_ENTRY (self->summary_entry)));
@@ -760,7 +811,7 @@ on_action_button_clicked_cb (GtkWidget *widget,
       gcal_event_set_description (self->event, note_text);
       g_free (note_text);
 
-      all_day = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->all_day_check));
+      all_day = gtk_switch_get_active (self->all_day_switch);
       was_all_day = gcal_event_get_all_day (self->event);
 
       if (!was_all_day && all_day)
@@ -821,6 +872,14 @@ on_action_button_clicked_cb (GtkWidget *widget,
       g_clear_pointer (&start_date, g_date_time_unref);
       g_clear_pointer (&end_date, g_date_time_unref);
 
+      /* Update alarms */
+      gcal_event_remove_all_alarms (self->event);
+
+      for (i = 0; i < self->alarms->len; i++)
+        gcal_event_add_alarm (self->event, g_ptr_array_index (self->alarms, i));
+
+      clear_alarms (self);
+
       /* Check Repeat popover and set recurrence-rules accordingly */
       old_recur = gcal_event_get_recurrence (self->event);
       freq = gtk_combo_box_get_active (GTK_COMBO_BOX (self->repeat_combo));
@@ -892,25 +951,10 @@ static void
 on_repeat_duration_changed_cb (GtkComboBox    *widget,
                                GcalEditDialog *self)
 {
-  switch (gtk_combo_box_get_active (widget))
-    {
-      case 0:
-        gtk_widget_hide (self->repeat_duration_stack);
-        break;
+  gint active = gtk_combo_box_get_active (widget);
 
-      case 1:
-        gtk_widget_show (self->repeat_duration_stack);
-        gtk_stack_set_visible_child (GTK_STACK (self->repeat_duration_stack), self->number_of_occurrences_spin);
-        break;
-
-      case 2:
-        gtk_widget_show (self->repeat_duration_stack);
-        gtk_stack_set_visible_child (GTK_STACK (self->repeat_duration_stack), self->until_date_selector);
-        break;
-
-      default:
-        break;
-    }
+  gtk_widget_set_visible (self->number_of_occurrences_spin, active == 1);
+  gtk_widget_set_visible (self->until_date_selector, active == 2);
 }
 
 static void
@@ -918,93 +962,63 @@ on_repeat_type_changed_cb (GtkComboBox    *combobox,
                            GcalEditDialog *self)
 {
   GcalRecurrenceFrequency frequency;
-  gboolean has_recurrence;
 
   frequency = gtk_combo_box_get_active (combobox);
-  has_recurrence = frequency != GCAL_RECURRENCE_NO_REPEAT;
 
-  gtk_widget_set_visible (self->repeat_limits_box, has_recurrence);
-
-  if (has_recurrence)
-    {
-      gtk_combo_box_set_active (GTK_COMBO_BOX (self->repeat_duration_combo), GCAL_RECURRENCE_FOREVER);
-      gtk_widget_show (self->repeat_duration_combo);
-      gtk_widget_hide (self->repeat_duration_stack);
-    }
+  gtk_combo_box_set_active (GTK_COMBO_BOX (self->repeat_duration_combo), GCAL_RECURRENCE_FOREVER);
+  gtk_widget_set_visible (self->repeat_duration_combo, frequency != GCAL_RECURRENCE_NO_REPEAT);
 }
 
 static void
-on_all_day_check_changed_cb (GtkToggleButton *button,
-                             GcalEditDialog  *self)
+on_all_day_switch_active_changed_cb (GtkSwitch      *all_day_switch,
+                                     GParamSpec     *pspec,
+                                     GcalEditDialog *self)
 {
-  gboolean active = gtk_toggle_button_get_active (button);
+  gboolean active = gtk_switch_get_active (all_day_switch);
 
   gtk_widget_set_sensitive (self->start_time_selector, !active);
   gtk_widget_set_sensitive (self->end_time_selector, !active);
+
+  update_date_labels (self);
 }
 
 static void
-on_remove_alarm_button_clicked (GtkButton *button,
-                                GtkWidget *row)
+on_remove_alarm_cb (GcalAlarmRow   *alarm_row,
+                    GcalEditDialog *self)
 {
   ECalComponentAlarm *alarm;
-  GcalEditDialog *self;
   GtkWidget *alarm_button;
-  GcalEvent *event;
   gint trigger_minutes;
+  gsize i;
 
-  self = GCAL_EDIT_DIALOG (gtk_widget_get_toplevel (row));
-  alarm = g_object_get_data (G_OBJECT (row), "alarm");
-  event = g_object_get_data (G_OBJECT (row), "event");
-  trigger_minutes = get_alarm_trigger_minutes (event, alarm);
+  GCAL_ENTRY;
 
-  /*
-   * Make the button sensitive again
-   */
+  alarm = gcal_alarm_row_get_alarm (alarm_row);
+  trigger_minutes = get_alarm_trigger_minutes (self->event, alarm);
+
+  /* Make the button sensitive again */
   alarm_button = get_row_for_alarm_trigger_minutes (self, trigger_minutes);
 
   if (alarm_button)
     gtk_widget_set_sensitive (alarm_button, TRUE);
 
-  gcal_event_remove_alarm (event, trigger_minutes);
+  /* Remove from the array */
+  for (i = 0; i < self->alarms->len; i++)
+    {
+      ECalComponentAlarm *a = g_ptr_array_index (self->alarms, i);
 
-  gcal_manager_update_event (gcal_context_get_manager (self->context),
-                             event,
-                             GCAL_RECURRENCE_MOD_THIS_ONLY);
+      if (trigger_minutes == get_alarm_trigger_minutes (self->event, a))
+        {
+          GCAL_TRACE_MSG ("Removed alarm for %d minutes", trigger_minutes);
 
-  gtk_widget_destroy (row);
+          g_ptr_array_remove_index (self->alarms, i);
+          break;
+        }
+    }
 
-  /*
-   * In order to not allocate a spacing between the listbox and the
-   * add alarms button, we should always keep the listbox:visible property
-   * updated.
-   */
-  gtk_widget_set_visible (self->alarms_listbox, gcal_event_has_alarms (self->event));
-}
-
-static void
-on_sound_toggle_changed_cb (GtkToggleButton *button,
-                            GtkWidget       *row)
-{
-  ECalComponentAlarmAction action;
-  ECalComponentAlarm *alarm;
-  GtkWidget *image;
-  gboolean has_sound;
-
-  alarm = g_object_get_data (G_OBJECT (row), "alarm");
-  image = gtk_bin_get_child (GTK_BIN (button));
-  has_sound = gtk_toggle_button_get_active (button);
-
-  /* Setup the alarm action */
-  action = has_sound ? E_CAL_COMPONENT_ALARM_AUDIO : E_CAL_COMPONENT_ALARM_DISPLAY;
-
-  e_cal_component_alarm_set_action (alarm, action);
-
-  /* Update the volume icon */
-  gtk_image_set_from_icon_name (GTK_IMAGE (image),
-                                has_sound ? "audio-volume-high-symbolic" : "audio-volume-muted-symbolic",
-                                GTK_ICON_SIZE_BUTTON);
+  gtk_container_remove (GTK_CONTAINER (self->alarms_listbox), GTK_WIDGET (alarm_row));
 
+  GCAL_EXIT;
 }
 
 static void
@@ -1018,58 +1032,71 @@ on_time_format_changed_cb (GcalEditDialog *self)
   gcal_time_selector_set_time_format (GCAL_TIME_SELECTOR (self->end_time_selector), time_format);
 }
 
+static GtkWidget *
+create_alarm_row (GcalEditDialog     *self,
+                  ECalComponentAlarm *alarm)
+{
+  GtkWidget *row;
+
+  row = gcal_alarm_row_new (alarm);
+  g_signal_connect_object (row, "remove-alarm", G_CALLBACK (on_remove_alarm_cb), self, 0);
+
+  return row;
+}
+
 static void
 setup_alarms (GcalEditDialog *self)
 {
-  GList *alarms, *l;
-  guint i;
+  g_autoptr (GList) alarms = NULL;
+  GList *l;
+  gsize i;
 
-  gtk_widget_set_visible (self->alarms_listbox, gcal_event_has_alarms (self->event));
+  GCAL_ENTRY;
 
-  alarms = gcal_event_get_alarms (self->event);
+  clear_alarms (self);
 
-  /* Remove previous alarms */
-  gtk_container_foreach (GTK_CONTAINER (self->alarms_listbox),
-                         (GtkCallback) gtk_widget_destroy,
-                         NULL);
-
-  /*
-   * We start by making all alarm buttons sensitive,
-   * and only make them insensitive when needed.
-   */
+  /* We start by making all alarm buttons sensitive, and only make them insensitive when needed */
   for (i = 0; i < G_N_ELEMENTS (minutes_button); i++)
     gtk_widget_set_sensitive (WIDGET_FROM_OFFSET (minutes_button[i].button_offset), TRUE);
 
+  alarms = gcal_event_get_alarms (self->event);
   for (l = alarms; l != NULL; l = l->next)
+    g_ptr_array_add (self->alarms, l->data);
+
+  for (i = 0; i < self->alarms->len; i++)
     {
+      ECalComponentAlarm *alarm;
       GtkWidget *row;
       gint minutes;
+      guint j;
 
-      row = create_row_for_alarm (self->event, l->data);
-
-      if (!row)
-        continue;
+      alarm = g_ptr_array_index (self->alarms, i);
 
       /* Make already-added alarm buttons insensitive */
-      minutes = get_alarm_trigger_minutes (self->event, l->data);
+      minutes = get_alarm_trigger_minutes (self->event, alarm);
 
-      for (i = 0; i < G_N_ELEMENTS (minutes_button); i++)
+      for (j = 0; j < G_N_ELEMENTS (minutes_button); j++)
         {
           if (minutes_button[i].minutes == minutes)
-            gtk_widget_set_sensitive (WIDGET_FROM_OFFSET (minutes_button[i].button_offset), FALSE);
+            gtk_widget_set_sensitive (WIDGET_FROM_OFFSET (minutes_button[j].button_offset), FALSE);
         }
 
+      GCAL_TRACE_MSG ("Adding alarm for %u minutes", minutes);
+
       /* Add the row */
+      row = create_alarm_row (self, alarm);
       gtk_container_add (GTK_CONTAINER (self->alarms_listbox), row);
     }
 
-  g_list_free (alarms);
+  GCAL_EXIT;
 }
 
 static void
 on_add_alarm_button_clicked_cb (GtkWidget      *button,
                                 GcalEditDialog *self)
 {
+  ECalComponentAlarm *alarm;
+  GtkWidget *row;
   guint i, minutes;
 
   /* Search for the button minute */
@@ -1087,22 +1114,25 @@ on_add_alarm_button_clicked_cb (GtkWidget      *button,
   if (minutes == 0)
     return;
 
-  /* Add the alarm */
-  gcal_event_add_alarm (self->event, minutes, FALSE);
+  alarm = create_alarm (minutes);
 
-  /*
-   * Instead of manually handling stuff, simply remove all alarms and
-   * add back again.
-   */
-  setup_alarms (self);
+  row = create_alarm_row (self, alarm);
+  gtk_container_add (GTK_CONTAINER (self->alarms_listbox), row);
+
+  g_ptr_array_add (self->alarms, alarm);
 
-  /*
-   * Since we don't allow more than 1 alarm per time, set the button
-   * to insensitive so it cannot be triggered anymore.
-   */
   gtk_widget_set_sensitive (button, FALSE);
 }
 
+static void
+on_alarms_listbox_row_activated_cb (GtkListBox     *alarms_listbox,
+                                    GtkListBoxRow  *row,
+                                    GcalEditDialog *self)
+{
+  if (row == self->new_alarm_row)
+    gtk_popover_popup (self->alarms_popover);
+}
+
 
 /*
  * Gobject overrides
@@ -1117,6 +1147,7 @@ gcal_edit_dialog_finalize (GObject *object)
 
   self = GCAL_EDIT_DIALOG (object);
 
+  g_clear_pointer (&self->alarms, g_ptr_array_unref);
   g_clear_object (&self->action_group);
   g_clear_object (&self->context);
   g_clear_object (&self->event);
@@ -1270,7 +1301,7 @@ gcal_edit_dialog_class_init (GcalEditDialogClass *klass)
 
   g_object_class_install_properties (object_class, N_PROPS, properties);
 
-  gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/edit-dialog.ui");
+  gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/ui/gui/gcal-edit-dialog.ui");
 
   /* Alarms */
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, five_minutes_button);
@@ -1281,6 +1312,8 @@ gcal_edit_dialog_class_init (GcalEditDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, two_days_button);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, three_days_button);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, one_week_button);
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, alarms_popover);
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, new_alarm_row);
   /* Buttons */
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, done_button);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, cancel_button);
@@ -1293,10 +1326,12 @@ gcal_edit_dialog_class_init (GcalEditDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, end_time_selector);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, end_date_selector);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, location_entry);
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, event_start_label);
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, event_end_label);
   /* Other */
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, alarms_listbox);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, notes_text);
-  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, all_day_check);
+  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, all_day_switch);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, titlebar);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, title_label);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, subtitle_label);
@@ -1304,8 +1339,6 @@ gcal_edit_dialog_class_init (GcalEditDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, number_of_occurrences_spin);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, repeat_combo);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, repeat_duration_combo);
-  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, repeat_duration_stack);
-  gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, repeat_limits_box);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, scrolled_window);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, source_image);
   gtk_widget_class_bind_template_child (widget_class, GcalEditDialog, sources_popover);
@@ -1317,7 +1350,8 @@ gcal_edit_dialog_class_init (GcalEditDialogClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, sync_datetimes);
   gtk_widget_class_bind_template_callback (widget_class, on_action_button_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_add_alarm_button_clicked_cb);
-  gtk_widget_class_bind_template_callback (widget_class, on_all_day_check_changed_cb);
+  gtk_widget_class_bind_template_callback (widget_class, on_alarms_listbox_row_activated_cb);
+  gtk_widget_class_bind_template_callback (widget_class, on_all_day_switch_active_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_repeat_duration_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_repeat_type_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_summary_entry_changed_cb);
@@ -1327,6 +1361,7 @@ gcal_edit_dialog_class_init (GcalEditDialogClass *klass)
 static void
 gcal_edit_dialog_init (GcalEditDialog *self)
 {
+  self->alarms = g_ptr_array_new_with_free_func (e_cal_component_alarm_free);
   self->writable = TRUE;
 
   gtk_widget_init_template (GTK_WIDGET (self));
@@ -1434,11 +1469,11 @@ gcal_edit_dialog_set_event (GcalEditDialog *self,
 
   if (frequency == GCAL_RECURRENCE_NO_REPEAT)
     {
-      gtk_widget_hide (self->repeat_limits_box);
+      gtk_widget_hide (self->repeat_duration_combo);
     }
   else
     {
-      gtk_widget_show (self->repeat_limits_box);
+      gtk_widget_show (self->repeat_duration_combo);
       gtk_widget_show (self->repeat_duration_combo);
     }
 
@@ -1518,7 +1553,9 @@ gcal_edit_dialog_set_event (GcalEditDialog *self,
   g_signal_handlers_unblock_by_func (self->end_time_selector, sync_datetimes, self);
 
   /* all_day  */
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->all_day_check), all_day);
+  gtk_switch_set_active (self->all_day_switch, all_day);
+
+  update_date_labels (self);
 
   /* recurrence_changed */
   self->recurrence_changed = FALSE;
